'  Copyright 2003-2004 Luke Plant L.Plant.98@cantab.net
'  
'  Redistribution and use in source and binary forms, with or without
'  modification, are permitted provided that the following conditions
'  are met:
'  
'  1. Redistributions of source code must retain the above copyright
'     notice, this list of conditions and the following disclaimer.
'  2. Redistributions in binary form must reproduce the above copyright
'     notice, this list of conditions and the following disclaimer in the
'     documentation and/or other materials provided with the distribution.
'  
'  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
'  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
'  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
'  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
'  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
'  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
'  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
'  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
'  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
'  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
'  
Attribute VB_Name = "modExcel"
Option Explicit

' Functions that are Excel/VBA functions, general convenience
' functions, and low level.

' Dependencies:
' - Require the modGeneric module
' - Require a function i18n() to do translation.  If no translation is needed,
'   i18n() can be defined as:
'     Function i18n(s as String) as String
'      i18n = s
'     End Function
' - GetWorkbookValue requires the workbook to have a cell named "GetWorkbookTemp"
' - MSGBOXTITLE should be defined (usually to the name of the project)

' Version history
' ---------------
'
' 0.1 Initial version


Function WorkbookIsOpen(str As String) As Boolean
    Dim wkbk As Workbook
    WorkbookIsOpen = False
    For Each wkbk In Workbooks
        If wkbk.Name = str Then
            WorkbookIsOpen = True
            Exit Function
        End If
    Next wkbk
End Function

Sub CloseAWorkbook(file As String)
    On Error Resume Next
    Workbooks(file).Close False
    On Error GoTo 0
End Sub

Function GetFirstEmptyRow(wks As Worksheet, col As Integer) As Integer
    Dim row As Integer
    row = 1
    Do Until IsEmpty(wks.Cells(row, col))
        row = row + 1
    Loop
    GetFirstEmptyRow = row
End Function

Function FileExists(fname As String) As Boolean
    FileExists = False
    On Error GoTo Fail
    If Dir(fname) <> "" Then
        FileExists = True
    End If
Fail:
End Function

' NB this returns false if the file was not saved for whatever reason.
' If there was a file error, the workbook will also be closed
Function SaveWithPromptedFilename(wkbk As Workbook, Name As String) As Boolean
    Dim RepeatLoop As Boolean, fname As Variant
    SaveWithPromptedFilename = True
    Application.DisplayAlerts = False
    Do
        RepeatLoop = False
        fname = Application.GetSaveAsFilename(Name)
        If fname = False Then
            ' pressed cancel
            SaveWithPromptedFilename = False
            wkbk.Close SaveChanges:=False
            Exit Function
        End If
        If Dir(fname) <> "" Then
            ' "File already exi
            If MsgBox(Printf(i18n("The file '%s' already exists.  Overwrite the file?"), fname), vbYesNo, MSGBOXTITLE) = vbNo Then
                RepeatLoop = True 'prompt again for file name
            End If
        End If
    Loop While RepeatLoop
    ThisWorkbook.Activate
    On Error GoTo Error_AlreadyOpen
    wkbk.SaveAs fname
    Application.DisplayAlerts = True
    Exit Function
    
Error_AlreadyOpen:

    Application.DisplayAlerts = True
    MsgBox i18n("TThere was a file error. You probably have a file with that name open.  Please close it before trying to create ano
ther workbook with the same name"), , MSGBOXTITLE
    wkbk.Close SaveChanges:=False
    SaveWithPromptedFilename = False
    On Error GoTo 0
    Exit Function
End Function

Sub PrintArray(arr, Optional indent As Integer = 0)
    ' prints 1-D arrays (recursively if an element is itself
    ' an array).
    Dim i As Integer
    If Not IsArray(arr) Then
        Debug.Print "Not an array!"
        Exit Sub
    End If
    For i = 0 To UBound(arr)
        If IsArray(arr(i)) Then
            Debug.Print Space(indent) & "[" & CStr(i) & "] => Array:"
            Debug.Print Space(indent) & "  {"
            PrintArray arr(i), indent + 3
            Debug.Print Space(indent) & "  }"
        Else
            Debug.Print Space(indent) & "[" & CStr(i) & "] => " & arr(i)
        End If
    Next i
End Sub

' RangeCommentF(rng) returns rng.Comment.Text, returning "" if
' rng.Comment doesn't exist
Function RangeCommentF(rng As Range) As String
    On Error GoTo NoComment
    RangeCommentF = rng.Comment.Text
    Exit Function
NoComment:
    RangeCommentF = ""
End Function

Function RangeLookup(val As Variant, rng As Range, colsearch As Integer, colreturn As Integer) As Variant
    ' search rng for val in column colsearch (col = 1 is the first column as per Excel)
    ' return data found in column colreturn, or Empty if match not found
    Dim pos As Integer
    On Error GoTo ExitFunc
    pos = WorksheetFunction.match(val, rng.Columns(colsearch), 0)
    RangeLookup = rng.Cells(pos, colreturn).value
    Exit Function
ExitFunc:
    RangeLookup = Empty
End Function

Function RangeExists(nm As String) As Boolean
    On Error GoTo badname
    If Range(nm).Name.Name = nm Then
        RangeExists = True
        Exit Function
    End If

badname:
    RangeExists = False

End Function

Public Function GetWorkbookValue(fname As String, wks As String, ref As String) As Variant
    ' Return a single value from a closed workbook
    ' Achieved by using a worksheet function in an invisible cell
    ' Fname must be a fully qualified path.
    
    Dim dirnm As String, i As Integer, tmp As Variant, file As String
    Dim wkst As Worksheet
    ScreenFreeze
    tmp = Split(fname, "\")
    file = tmp(UBound(tmp))
    ArrayPop tmp
    dirnm = Join(tmp, "\") & "\"
    On Error GoTo Error_1
    [GetWorkbookTemp].NumberFormat = ""
    [GetWorkbookTemp].Formula = "='" & dirnm & "[" & file & "]" & wks & "'!" & ref
    Set wkst = Names("GetWorkbookTemp").RefersToRange.Worksheet
    If wkst.EnableCalculation = False Then
        wkst.EnableCalculation = True
        wkst.Calculate
        GetWorkbookValue = [GetWorkbookTemp].value
        wkst.EnableCalculation = False
    Else
        GetWorkbookValue = [GetWorkbookTemp].value
    End If
    [GetWorkbookTemp] = ""
    ScreenThaw
    Exit Function
    
Error_1:
    ScreenThaw
    [GetWorkbookTemp] = ""
    GetWorkbookValue = Null
End Function

Sub AddToRange(rng As Range, newrange As Range)
    If rng Is Nothing Then
        Set rng = newrange
    Else
        Set rng = Union(rng, newrange)
    End If
End Sub


