'  Copyright 2003-2004 Luke Plant L.Plant.98@cantab.net
'  
'  Redistribution and use in source and binary forms, with or without
'  modification, are permitted provided that the following conditions
'  are met:
'  
'  1. Redistributions of source code must retain the above copyright
'     notice, this list of conditions and the following disclaimer.
'  2. Redistributions in binary form must reproduce the above copyright
'     notice, this list of conditions and the following disclaimer in the
'     documentation and/or other materials provided with the distribution.
'  
'  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
'  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
'  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
'  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
'  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
'  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
'  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
'  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
'  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
'  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
'  
Attribute VB_Name = "modRegistry"
Option Explicit

' Module for reading, writing and creating registry values/keys
' Currently only support REG_DWORD and REG_SZ types, where the
' type is determined by the Vartype() of the input value for writing

' Setting a value does not create a key automatically - you must
' use RegistryKeyExists() and RegistryCreateKey()

Declare Function RegQueryValueEx Lib "advapi32.dll" Alias "RegQueryValueExA" (ByVal pRegKey As Long, ByVal ValueName As String, ByVal Reserved As Long, ValueType As Long, ValueData As Any, DataLen As Long) As Long
Declare Function RegOpenKeyEx Lib "advapi32.dll" Alias "RegOpenKeyExA" (ByVal HKEYROOT As Long, ByVal SubKey As String, ByVal ulOptions As Long, ByVal samDesired As Long, ByRef pRegKey As Long) As Long
Declare Function RegCloseKey Lib "advapi32.dll" (ByVal pRegKey As Long) As Long
Declare Function RegSetValueEx Lib "advapi32.dll" Alias "RegSetValueExA" (ByVal pRegKey As Long, ByVal ValueName As String, ByVal Reserved As Long, ByVal ValueType As Long, ValueData As Any, ByVal DataLen As Long) As Long
Declare Function RegCreateKeyEx Lib "advapi32.dll" Alias "RegCreateKeyExA" (ByVal HKEYROOT As Long, ByVal SubKey As String, ByVal Reserved As Long, ByVal lpClass As String, ByVal Options As Long, ByVal samDesired As Long, lpSecurityAttributes As SECURITY_ATTRIBUTES, pRegKey As Long, lpdwDisposition As Long) As Long

Public Const HKEY_CLASSES_ROOT = &H80000000
Public Const HKEY_CURRENT_USER = &H80000001
Public Const HKEY_LOCAL_MACHINE = &H80000002
Public Const HKEY_USERS = &H80000003
Public Const HKEY_PERFORMANCE_DATA = &H80000004
Public Const HKEY_CURRENT_CONFIG = &H80000005
Public Const HKEY_DYN_DATA = &H80000006

Const REG_NONE = 0
Const REG_SZ = 1
Const REG_EXPAND_SZ = 2
Const REG_BINARY = 3
Const REG_DWORD = 4
Const REG_DWORD_LITTLE_ENDIAN = 4
Const REG_DWORD_BIG_ENDIAN = 5
Const REG_LINK = 6
Const REG_MULTI_SZ = 7
Const REG_RESOURCE_LIST = 8

Const KEY_ALL_ACCESS = &HF003F
Const KEY_CREATE_LINK = &H20
Const KEY_CREATE_SUB_KEY = &H4
Const KEY_ENUMERATE_SUB_KEYS = &H8
Const KEY_EXECUTE = &H20019
Const KEY_NOTIFY = &H10
Const KEY_QUERY_VALUE = &H1
Const KEY_READ = &H20019
Const KEY_SET_VALUE = &H2
Const KEY_WRITE = &H20006

Const DATA_MAX_SIZE = 65535
Const DATA_MIN_SIZE = 256
Const DATA_STEP_SIZE = 512

Private Type SECURITY_ATTRIBUTES
    nLength As Long
    lpSecurityDescriptor As Long
    bInheritHandle As Integer
End Type
 
Public Function RegistryGetValue(HKEYROOT As Long, SubKey As String, ValueName As String) As Variant
    Dim DataBuf As String
    Dim DataLen As Long
    Dim DataReqSize As Long
    Dim pRegKey As Long
    Dim ValueType As Long

    If RegOpenKeyEx(HKEYROOT, SubKey, 0, KEY_QUERY_VALUE, pRegKey) <> 0 Then
        RegistryGetValue = Null
        Exit Function
    End If
    
    DataReqSize = DATA_MIN_SIZE
    DataLen = DataReqSize
    DataBuf = Space(DataLen)

    Do Until RegQueryValueEx(pRegKey, ValueName, 0, ValueType, ByVal DataBuf, DataLen) = 0
        ' the above line could have failed due to insufficient size,
        ' in which case DataLen will contain the required size
        If DataLen > DataReqSize Then
            DataReqSize = DataReqSize + DATA_STEP_SIZE
            DataLen = DataReqSize
            DataBuf = Space(DataLen)
        Else
            RegistryGetValue = Null
            RegCloseKey pRegKey
            Exit Function
        End If
    Loop
    Select Case ValueType
        Case REG_DWORD
            RegistryGetValue = _
                Asc(Mid(DataBuf, 4, 1)) * (2 ^ 24) + _
                Asc(Mid(DataBuf, 3, 1)) * (2 ^ 16) + _
                Asc(Mid(DataBuf, 2, 1)) * (2 ^ 8) + _
                Asc(Mid(DataBuf, 1, 1))
        Case REG_SZ
            RegistryGetValue = Left(DataBuf, DataLen - 1)
        Case Else
            RegistryGetValue = Null
    End Select
    
    RegCloseKey pRegKey
End Function

Public Function RegistrySetValue(HKEYROOT As Long, SubKey As String, ValueName As String, ValueVal As Variant) As Boolean
    Dim pRegKey As Long
    Dim ValueType As Long

    Dim RegDWord As Long
    Dim RegSZ As String
    
    ValueType = REG_NONE
    Select Case VarType(ValueVal)
        Case vbBoolean, vbInteger, vbLong, vbByte
            ValueType = REG_DWORD
            RegDWord = CLng(ValueVal)
        Case vbString
            ValueType = REG_SZ
            RegSZ = ValueVal
        Case vbSingle, vbDouble
            ValueType = REG_SZ
            RegSZ = str(ValueVal)
        Case vbDate
            ValueType = REG_SZ
            RegSZ = str(CLng(ValueVal))
    End Select
    If ValueType = REG_NONE Then
        RegistrySetValue = False
        Exit Function
    End If
    
    If RegOpenKeyEx(HKEYROOT, SubKey, 0, KEY_SET_VALUE, pRegKey) <> 0 Then
        RegistrySetValue = False
        Exit Function
    End If
    
    Select Case ValueType
        Case REG_SZ
            If RegSetValueEx(pRegKey, ValueName, 0, ValueType, ByVal RegSZ, Len(RegSZ)) = 0 Then
                RegistrySetValue = True
            Else
                RegistrySetValue = False
            End If
        Case REG_DWORD
            If RegSetValueEx(pRegKey, ValueName, 0, ValueType, RegDWord, Len(RegDWord)) = 0 Then
                RegistrySetValue = True
            Else
                RegistrySetValue = False
            End If
        Case Else
            RegistrySetValue = False

    End Select
    RegCloseKey pRegKey
    
End Function

Public Function RegistryCreateKey(HKEYROOT As Long, SubKey As String) As Boolean
    Dim attr As SECURITY_ATTRIBUTES
    Dim disp As Long
    Dim pRegKey As Long
    attr.nLength = Len(attr)
    attr.bInheritHandle = 1
    attr.lpSecurityDescriptor = 0
    If RegCreateKeyEx(HKEYROOT, SubKey, 0, "", 0, KEY_ALL_ACCESS, attr, pRegKey, disp) = 0 Then
        RegistryCreateKey = True
        RegCloseKey pRegKey
    Else
        RegistryCreateKey = False
    End If
    
End Function

' will also return false if permission is denied
Public Function RegistryKeyExists(HKEYROOT As Long, SubKey As String) As Boolean
    Dim pRegKey As Long
    If RegOpenKeyEx(HKEYROOT, SubKey, 0, KEY_READ, pRegKey) = 0 Then
        RegistryKeyExists = True
        RegCloseKey pRegKey
    Else
        RegistryKeyExists = False
    End If
End Function

